package com.yunzhi.shigongli.service.impl;

import com.github.binarywang.wxpay.bean.notify.WxPayOrderNotifyResult;
import com.github.binarywang.wxpay.bean.order.WxPayMpOrderResult;
import com.github.binarywang.wxpay.bean.result.WxPayRefundResult;
import com.github.binarywang.wxpay.constant.WxPayConstants;
import com.yunzhi.shigongli.common.*;
import com.yunzhi.shigongli.entity.*;
import com.yunzhi.shigongli.mapper.*;
import com.yunzhi.shigongli.service.ITaOrderService;

import com.yunzhi.shigongli.vo.OrderItem;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDateTime;
import java.util.List;

/**
 * <p>
 * 订单表 服务实现类
 * </p>
 *
 * @author yansen
 * @since 2021-08-17
 */
@Slf4j
@Service
public class TaOrderServiceImpl extends BaseServiceImpl<TaOrderMapper, TaOrder> implements ITaOrderService {

    @Autowired
    WxPayUtils wxPayUtils;

    @Autowired
    TaShopPackageMapper taShopPackageMapper;

    @Autowired
    TaShopPackageSettingMapper packageSettingMapper;

    @Autowired
    TaOrderMapper taOrderMapper;

    @Autowired
    TaOrderSubMapper taOrderSubMapper;

    @Autowired
    TaOrderCommissionMapper commissionMapper;

    @Autowired
    TaOrderPayMapper orderPayMapper;

    /**
     * 下单支付
     * @param person
     * @param orderItems
     * @return
     * @throws Exception
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public WxPayMpOrderResult order(TaPerson person, OrderItem[] orderItems) throws Exception {
        LocalDateTime now = LocalDateTime.now();

        // 先生成主订单
        TaOrder taOrder = new TaOrder();
        taOrder.setOrderId(getOrderNo());
        taOrder.setDescription("套餐");
        taOrder.setPersonId(person.getPersonId());
        // 默认未支付
        taOrder.setStatus(Constants.ORDER_READY);
        taOrder.setCreateDate(now);

        // 子订单列表
        Integer charge = createAndSaveOrderSubList(taOrder, orderItems);
        taOrder.setCharges(charge);
        save(taOrder);

        // 发起微信支付订单
        return wxPayUtils.createOrder(person, taOrder);
    }

    /**
     * 取消订单
     * @param taOrder
     * @param taPerson
     * @throws Exception
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public void cancelOrder(TaOrder taOrder, TaPerson taPerson) throws Exception {
        List<TaOrderSub> orderSubList = taOrderSubMapper.getListByOrder(taOrder.getOrderId());
        if (null == orderSubList || orderSubList.size() < 1) {
            throw new Exception("未找到订单详情");
        }

        // 如果有已核销的, 则不能取消
        Object[] o1 = orderSubList.stream().filter(x -> x.getIsVerified()).toArray();
        if (o1.length > 0) {
            throw new Exception("订单存在已核销内容, 不能取消");
        }

        // 如果已支付, 则不能取消
        if (Constants.ORDER_PAYED.equals(taOrder.getStatus())) {
            throw new Exception("订单已支付, 请先申请退款");
        }

        // 只有未支付的, 或者已退款的可以取消
        if (!Constants.ORDER_READY.equals(taOrder.getStatus())
            && !Constants.ORDER_REFUND.equals(taOrder.getStatus())) {
            log.error("退款订单【{}】, 状态为 {}", taOrder.getOrderId(), taOrder.getStatus());
            throw new Exception("订单状态异常, 请联系相关人员");
        }

        // 只需要更新状态即可
        taOrderMapper.updateStatus(taOrder.getOrderId(), Constants.ORDER_CANCEL);
        taOrderSubMapper.updateStatusByOrder(taOrder.getOrderId(), Constants.ORDER_CANCEL);
    }

    /**
     * 获取订单号
     * @return
     */
    private String getOrderNo() {
        LocalDateTime now = LocalDateTime.now();
        String today = DateUtils.toString(now, "yyyyMMddHHmm");
        String orderNo = null;
        do {
            String random = StringUtils.random(4);
            orderNo = String.format("%s-%s", today, random);
            TaOrder taOrder = getById(orderNo);
            if (null == taOrder) {
                break;
            }
        } while (true);

        return orderNo;
    }

    /**
     * 生成并保存子订单
     * @param taOrder
     * @param orderItems
     * @return
     * @throws Exception
     */
    private Integer createAndSaveOrderSubList(TaOrder taOrder, OrderItem[] orderItems) throws Exception {
        LocalDateTime now = LocalDateTime.now();

        Integer totalCharge = 0;
        for (int i = 0; i < orderItems.length; i += 1) {
            OrderItem item = orderItems[i];
            String subOrderId = taOrder.getOrderId() + "-" + (i + 1);
            TaOrderSub orderSub = new TaOrderSub();
            orderSub.setSubOrderId(subOrderId);
            orderSub.setOrderId(taOrder.getOrderId());
            // 默认未核销
            orderSub.setIsVerified(false);
            // 未支付
            orderSub.setStatus(Constants.ORDER_READY);
            // 时间
            orderSub.setCreateDate(now);

            if (StringUtils.isEmpty(item.getItemId())) {
                throw new Exception("订单中存在空的订单项");
            }
            if (null == item.getPrice() || item.getPrice() < 0) {
                throw new Exception("订单中存在不正常的金额");
            }
            if (null == item.getAmount() || item.getAmount() < 0) {
                throw new Exception("订单中存在不正常的数量");
            }

            // 套餐ID
            String packageId = item.getItemId();
            if (StringUtils.isEmpty(packageId)) {
                throw new Exception("订单中存在空的订单项");
            }

            // 套餐
            TaShopPackage shopPackage = taShopPackageMapper.selectById(packageId);
            if (null == shopPackage || !Constants.STATUS_NORMAL.equals(shopPackage.getStatus())) {
                throw new Exception("订单中存在异常状态套餐");
            }
            if (shopPackage.getCashback() >= shopPackage.getActualPrice()) {
                throw new Exception("订单中存在异常套餐, 请联系管理员");
            }

            orderSub.setPackageId(packageId);

            // 价格
            if (!item.getPrice().equals(shopPackage.getActualPrice())) {
                throw new Exception("订单中存在变化的价格, 请刷新重试");
            }

            orderSub.setUnitPrice(item.getPrice());
            orderSub.setCharges(item.getPrice() * item.getAmount());
            totalCharge += orderSub.getCharges();

            // 数量
            orderSub.setAmount(item.getAmount());

            // 返现设置
            orderSub.setCashback(shopPackage.getCashback());

            // 套餐分成设置
            TaShopPackageSetting packageSetting = packageSettingMapper.selectById(packageId);
            if (null != packageSetting) {
                // 引流分成
                Integer trafficRate = packageSetting.getTrafficRate();
                Integer trafficCommission = orderSub.getCharges() * (trafficRate / 1000);
                // 平台分成
                Integer platformRate = packageSetting.getPlatformRate();
                Integer platformCommission = orderSub.getCharges() * (platformRate / 1000);
                // 总提成
                Integer commissionCharge = trafficCommission + platformCommission;
                if (commissionCharge + orderSub.getCashback() >= orderSub.getCharges()) {
                    log.error("下单失败; 套餐[{}]抽成与返现设置有问题", packageId);
                    throw new Exception("订单中存在异常套餐, 请联系相关人员");
                }

                if (trafficCommission > 0) {
                    createAndSaveOrderCommission(orderSub, trafficCommission, Constants.COMMISSION_TRAFFIC);
                }

                if (platformCommission > 0) {
                    createAndSaveOrderCommission(orderSub, trafficCommission, Constants.COMMISSION_PLATFORM);
                }

                orderSub.setCommissionCharge(commissionCharge);
            } else {
                orderSub.setCommissionCharge(0);
            }

            taOrderSubMapper.insert(orderSub);
        }

        return totalCharge;
    }

    /**
     * 生成并保存抽成表
     * @param orderSub
     * @param charge
     * @param commissionType
     * @throws Exception
     */
    private void createAndSaveOrderCommission(TaOrderSub orderSub, Integer charge, String commissionType) throws Exception {
        LocalDateTime now = LocalDateTime.now();
        TaOrderCommission orderCommission = new TaOrderCommission();
        orderCommission.setOrderId(orderSub.getOrderId());
        orderCommission.setSubOrderId(orderSub.getSubOrderId());
        orderCommission.setCommissionType(commissionType);
        orderCommission.setCharge(charge);
        orderCommission.setCreateDate(now);
        commissionMapper.insert(orderCommission);
    }
}
